"""URI normalize.

Copyright (c) 2020 Nikolay Panov
SPDX-License-Identifier: MIT

"""

from .url_normalize import url_normalize

__license__ = "MIT"
__version__ = "2.2.1"

__all__ = ["url_normalize"]
